import { ASTBase } from 'miniscript-core';
import { HandlerContainer } from '../handler-container';
import { Stack } from '../utils/stack';
import { Instruction, SourceLocation } from './instruction';
import { Module } from './module';
export interface ContextOptions {
    target: string;
    handler: HandlerContainer;
    debugMode?: boolean;
}
export type ContextInstruction = Partial<Instruction> & Required<Pick<Instruction, 'op'>> & {
    goto?: ContextInstruction;
};
export declare class Context {
    protected _target: Stack<string>;
    protected _handler: HandlerContainer;
    protected _module: Stack<Module>;
    protected _debugMode: boolean;
    protected _imports: Map<string, Instruction[]>;
    constructor(options: ContextOptions);
    get target(): Stack<string>;
    get module(): Stack<Module>;
    get handler(): HandlerContainer;
    get imports(): Map<string, Instruction[]>;
    isDebugMode(): boolean;
    pushCode(intstruction: ContextInstruction, node: ASTBase, name?: string): void;
    pushInternalCode(intstruction: ContextInstruction): void;
    getSourceLocation(node: ASTBase, name?: string): SourceLocation;
    getInternalLocation(): SourceLocation;
}
