"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Context = void 0;
const stack_1 = require("../utils/stack");
const module_1 = require("./module");
class Context {
    constructor(options) {
        var _a;
        this._target = new stack_1.Stack(options.target);
        this._module = new stack_1.Stack(new module_1.Module(options.target));
        this._handler = options.handler;
        this._imports = new Map();
        this._debugMode = (_a = options.debugMode) !== null && _a !== void 0 ? _a : false;
    }
    get target() {
        return this._target;
    }
    get module() {
        return this._module;
    }
    get handler() {
        return this._handler;
    }
    get imports() {
        return this._imports;
    }
    isDebugMode() {
        return this._debugMode;
    }
    pushCode(intstruction, node, name) {
        intstruction.source = this.getSourceLocation(node, name);
        this.module.peek().pushCode(intstruction);
    }
    pushInternalCode(intstruction) {
        intstruction.source = this.getInternalLocation();
        this.module.peek().pushCode(intstruction);
    }
    getSourceLocation(node, name) {
        const target = this.target.peek();
        return {
            name: name !== null && name !== void 0 ? name : node.type,
            path: target,
            start: node.start,
            end: node.end
        };
    }
    getInternalLocation() {
        return {
            name: 'internal',
            path: 'internal',
            start: { line: 0, character: 0 },
            end: { line: 0, character: 0 }
        };
    }
}
exports.Context = Context;
